@@ -21,6 +21,7 @@
 
 #include "kvu_utils.h" /* kvu_sleep() */
 
+#include "eca-session.h"
 #include "eca-control.h"
 #include "eca-test-case.h"
 
@@ -50,7 +51,7 @@ private:
 
 void ECA_CONTROL_TEST::do_run(void)
 {
-  cerr << "libecasound_tester: eca-control - chainsetup creation stress test" << endl;
+  cout << "libecasound_tester: eca-control - chainsetup creation stress test" << endl;
   do_run_chainsetup_creation();
 }
 
@@ -59,10 +60,10 @@ void ECA_CONTROL_TEST::do_run_chainsetup_creation(void)
   ECA_SESSION *esession = new ECA_SESSION();
   ECA_CONTROL *ectrl = new ECA_CONTROL(esession);
 
-  int iterations = 127;
+  int iterations = 15;
 
   for(int i = 0; i < iterations; i++) {
-    cerr << "libecasound_tester: do_run_chainsetup_creation() iteration " 
+    cout << "libecasound_tester: do_run_chainsetup_creation() iteration " 
 	 << i + 1 << " of " << iterations << "." << endl;
 
     ectrl->add_chainsetup("default");
@@ -75,7 +76,7 @@ void ECA_CONTROL_TEST::do_run_chainsetup_creation(void)
     ectrl->add_chain_operator("-ea:100");
     if (ectrl->selected_chain_operator() < 0 ||
 	ectrl->selected_chain_operator() > 1) ECA_TEST_FAILURE("Chain operator addition failed.");
-    ectrl->connect_chainsetup();
+    ectrl->connect_chainsetup(0);
     if (ectrl->is_connected() != true) ECA_TEST_FAILURE("Chainsetup connection failed.");
     ectrl->start();
     kvu_sleep(0, 200000000); /* 200ms */
