@@ -1,6 +1,6 @@
 // ------------------------------------------------------------------------
 // audioio-arts.cpp: Interface for communicating with aRts/MCOP.
-// Copyright (C) 2000 Kai Vehmanen (kaiv@wakkanet.fi)
+// Copyright (C) 2000,2002 Kai Vehmanen
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -17,27 +17,55 @@
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
 // ------------------------------------------------------------------------
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
+#include "audioio-device.h"
+#include "eca-logger.h"
+#include "eca-version.h"
 
-#include "audioio-types.h"
-#include "eca-debug.h"
+extern "C" {
+#include <artsc.h>
+}
 
-#ifdef COMPILE_ARTS
 #include "audioio_arts.h"
 
+#ifdef ECA_ENABLE_AUDIOIO_PLUGINS
+static const char* audio_io_keyword_const = "arts";
+static const char* audio_io_keyword_regex_const = "^arts$";
+
+const char* audio_io_keyword(void){return(audio_io_keyword_const); }
+const char* audio_io_keyword_regex(void){return(audio_io_keyword_regex_const); }
+int audio_io_interface_version(void) { return(ecasound_library_version_current); }
+#endif
+
 int ARTS_INTERFACE::ref_rep = 0;
 
 ARTS_INTERFACE::ARTS_INTERFACE(const string& name)
 {
-  label(name);
+  set_label(name);
+}
+
+ARTS_INTERFACE::~ARTS_INTERFACE(void)
+{
+  if (is_open() == true && is_running()) stop();
+
+  if (is_open() == true) {
+    close();
+  }
+
+  --ref_rep;
+  if (ref_rep == 0) ::arts_free();
+}
+
+ARTS_INTERFACE* ARTS_INTERFACE::clone(void) const
+{
+  ARTS_INTERFACE* target = new ARTS_INTERFACE();
+  for(int n = 0; n < number_of_params(); n++) {
+    target->set_parameter(n + 1, get_parameter(n + 1));
+  }
+  return(target);
 }
 
 void ARTS_INTERFACE::open(void) throw(AUDIO_IO::SETUP_ERROR&)
 {
-  if (is_open() == true) return;
-
   if (ref_rep == 0) {
     int err = ::arts_init();
     if (err < 0) {
@@ -59,30 +87,35 @@ void ARTS_INTERFACE::open(void) throw(AUDIO_IO::SETUP_ERROR&)
   ::arts_stream_set(stream_rep, ARTS_P_BUFFER_SIZE, buffersize() * frame_size());
   ::arts_stream_set(stream_rep, ARTS_P_BLOCKING, 1);
   samples_rep = 0;
-  toggle_open_state(true);
+
+  double total_latency = ::arts_stream_get(stream_rep, ARTS_P_TOTAL_LATENCY);
+  latency_rep = static_cast<long int>(total_latency * samples_per_second() / 1000.0f);
+  
+  AUDIO_IO::open();
 }
 
-void ARTS_INTERFACE::stop(void) { 
+void ARTS_INTERFACE::stop(void)
+{
   AUDIO_IO_DEVICE::stop();
 }
 
 void ARTS_INTERFACE::close(void)
 {
-  toggle_open_state(false);
   ::arts_close_stream(stream_rep);
+
+  AUDIO_IO::close();
 }
 
-void ARTS_INTERFACE::start(void) { 
+void ARTS_INTERFACE::start(void)
+{ 
   AUDIO_IO_DEVICE::start();
 }
 
-long ARTS_INTERFACE::position_in_samples(void) const { return(samples_rep); }
 long int ARTS_INTERFACE::read_samples(void* target_buffer, 
 				      long int samples) 
 {
   long int res = ::arts_read(stream_rep, target_buffer, frame_size() * samples);
   if (res >= 0) {
-    samples_rep += res;
     return(res / frame_size());
   }
   else {
@@ -93,13 +126,5 @@ long int ARTS_INTERFACE::read_samples(void* target_buffer,
 void ARTS_INTERFACE::write_samples(void* target_buffer, 
 				   long int samples) 
 {
-  samples_rep += ::arts_write(stream_rep, target_buffer, frame_size() * samples);
+  ::arts_write(stream_rep, target_buffer, frame_size() * samples);
 }
-
-ARTS_INTERFACE::~ARTS_INTERFACE(void) 
-{ 
-  --ref_rep;
-  if (ref_rep == 0) ::arts_free();
-}
-
-#endif // COMPILE_ARTS
